package com.sun.jmx.snmp;

import java.io.Serializable;

public class SnmpVarBind implements SnmpDataTypeEnums, Cloneable, Serializable
{
    private static final long serialVersionUID = 491778383240759376L;
    private static final String[] statusLegend;
    public static final int stValueUnspecified = 1;
    public static final int stValueOk = 2;
    public static final int stValueNoSuchObject = 3;
    public static final int stValueNoSuchInstance = 4;
    public static final int stValueEndOfMibView = 5;
    public static final SnmpNull noSuchObject;
    public static final SnmpNull noSuchInstance;
    public static final SnmpNull endOfMibView;
    public SnmpOid oid;
    public SnmpValue value;
    public int status;
    
    public SnmpVarBind() {
        this.oid = null;
        this.value = null;
        this.status = 1;
    }
    
    public SnmpVarBind(SnmpOid oid) {
        this.oid = null;
        this.value = null;
        this.status = 1;
        this.oid = oid;
    }
    
    public SnmpVarBind(SnmpOid oid, SnmpValue snmpValue) {
        this.oid = null;
        this.value = null;
        this.status = 1;
        this.oid = oid;
        this.setSnmpValue(snmpValue);
    }
    
    public SnmpVarBind(String s) throws SnmpStatusException {
        this.oid = null;
        this.value = null;
        this.status = 1;
        if (s.startsWith(".")) {
            this.oid = new SnmpOid(s);
        }
        else {
            try {
                this.handleLong(s, s.indexOf(46));
                this.oid = new SnmpOid(s);
            }
            catch (NumberFormatException ex) {
                int index = s.indexOf(46);
                if (index <= 0) {
                    this.oid = new SnmpOid(this.resolveVarName(s).getName());
                }
                else {
                    this.oid = new SnmpOid(this.resolveVarName(s.substring(0, index)).getName() + s.substring(index));
                }
            }
        }
    }
    
    public final SnmpOid getOid() {
        return this.oid;
    }
    
    public final void setOid(SnmpOid oid) {
        this.oid = oid;
        this.clearValue();
    }
    
    public final synchronized SnmpValue getSnmpValue() {
        return this.value;
    }
    
    public final void setSnmpValue(SnmpValue value) {
        this.value = value;
        this.setValueValid();
    }
    
    public final SnmpCounter64 getSnmpCounter64Value() throws ClassCastException {
        return (SnmpCounter64)this.value;
    }
    
    public final void setSnmpCounter64Value(long n) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpCounter64(n);
        this.setValueValid();
    }
    
    public final SnmpInt getSnmpIntValue() throws ClassCastException {
        return (SnmpInt)this.value;
    }
    
    public final void setSnmpIntValue(long n) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpInt(n);
        this.setValueValid();
    }
    
    public final SnmpCounter getSnmpCounterValue() throws ClassCastException {
        return (SnmpCounter)this.value;
    }
    
    public final void setSnmpCounterValue(long n) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpCounter(n);
        this.setValueValid();
    }
    
    public final SnmpGauge getSnmpGaugeValue() throws ClassCastException {
        return (SnmpGauge)this.value;
    }
    
    public final void setSnmpGaugeValue(long n) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpGauge(n);
        this.setValueValid();
    }
    
    public final SnmpTimeticks getSnmpTimeticksValue() throws ClassCastException {
        return (SnmpTimeticks)this.value;
    }
    
    public final void setSnmpTimeticksValue(long n) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpTimeticks(n);
        this.setValueValid();
    }
    
    public final SnmpOid getSnmpOidValue() throws ClassCastException {
        return (SnmpOid)this.value;
    }
    
    public final void setSnmpOidValue(String s) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpOid(s);
        this.setValueValid();
    }
    
    public final SnmpIpAddress getSnmpIpAddressValue() throws ClassCastException {
        return (SnmpIpAddress)this.value;
    }
    
    public final void setSnmpIpAddressValue(String s) throws IllegalArgumentException {
        this.clearValue();
        this.value = new SnmpIpAddress(s);
        this.setValueValid();
    }
    
    public final SnmpString getSnmpStringValue() throws ClassCastException {
        return (SnmpString)this.value;
    }
    
    public final void setSnmpStringValue(String s) {
        this.clearValue();
        this.value = new SnmpString(s);
        this.setValueValid();
    }
    
    public final SnmpOpaque getSnmpOpaqueValue() throws ClassCastException {
        return (SnmpOpaque)this.value;
    }
    
    public final void setSnmpOpaqueValue(byte[] array) {
        this.clearValue();
        this.value = new SnmpOpaque(array);
        this.setValueValid();
    }
    
    public final SnmpStringFixed getSnmpStringFixedValue() throws ClassCastException {
        return (SnmpStringFixed)this.value;
    }
    
    public final void setSnmpStringFixedValue(String s) {
        this.clearValue();
        this.value = new SnmpStringFixed(s);
        this.setValueValid();
    }
    
    public SnmpOidRecord resolveVarName(String s) throws SnmpStatusException {
        SnmpOidTable snmpOidTable = SnmpOid.getSnmpOidTable();
        if (snmpOidTable == null) {
            throw new SnmpStatusException(2);
        }
        if (s.indexOf(46) < 0) {
            return snmpOidTable.resolveVarName(s);
        }
        return snmpOidTable.resolveVarOid(s);
    }
    
    public final int getValueStatus() {
        return this.status;
    }
    
    public final String getValueStatusLegend() {
        return SnmpVarBind.statusLegend[this.status];
    }
    
    public final boolean isValidValue() {
        return this.status == 2;
    }
    
    public final boolean isUnspecifiedValue() {
        return this.status == 1;
    }
    
    public final void clearValue() {
        this.value = null;
        this.status = 1;
    }
    
    public final boolean isOidEqual(SnmpVarBind snmpVarBind) {
        return this.oid.equals(snmpVarBind.oid);
    }
    
    public final void addInstance(long n) {
        this.oid.append(n);
    }
    
    public final void addInstance(long[] array) throws SnmpStatusException {
        this.oid.addToOid(array);
    }
    
    public final void addInstance(String s) throws SnmpStatusException {
        if (s != null) {
            this.oid.addToOid(s);
        }
    }
    
    public void insertInOid(int n) {
        this.oid.insert(n);
    }
    
    public void appendInOid(SnmpOid snmpOid) {
        this.oid.append(snmpOid);
    }
    
    public final synchronized boolean hasVarBindException() {
        switch (this.status) {
            case 1:
            case 3:
            case 4:
            case 5: {
                return true;
            }
            default: {
                return false;
            }
        }
    }
    
    public void copyValueAndOid(SnmpVarBind snmpVarBind) {
        this.setOid((SnmpOid)(SnmpOid)snmpVarBind.oid.clone());
        this.copyValue(snmpVarBind);
    }
    
    public void copyValue(SnmpVarBind snmpVarBind) {
        if (snmpVarBind.isValidValue()) {
            this.value = snmpVarBind.getSnmpValue().duplicate();
            this.setValueValid();
        }
        else {
            this.status = snmpVarBind.getValueStatus();
            if (this.status == 5) {
                this.value = SnmpVarBind.endOfMibView;
            }
            else if (this.status == 3) {
                this.value = SnmpVarBind.noSuchObject;
            }
            else if (this.status == 4) {
                this.value = SnmpVarBind.noSuchInstance;
            }
        }
    }
    
    public Object cloneWithoutValue() {
        return new SnmpVarBind((SnmpOid)this.oid.clone());
    }
    
    public SnmpVarBind clone() {
        SnmpVarBind snmpVarBind = new SnmpVarBind();
        snmpVarBind.copyValueAndOid(this);
        return snmpVarBind;
    }
    
    public final String getStringValue() {
        return this.value.toString();
    }
    
    public final void setNoSuchObject() {
        this.value = SnmpVarBind.noSuchObject;
        this.status = 3;
    }
    
    public final void setNoSuchInstance() {
        this.value = SnmpVarBind.noSuchInstance;
        this.status = 4;
    }
    
    public final void setEndOfMibView() {
        this.value = SnmpVarBind.endOfMibView;
        this.status = 5;
    }
    
    @Override
    public final String toString() {
        StringBuilder sb = new StringBuilder(400);
        sb.append("Object ID : ").append(this.oid.toString());
        if (this.isValidValue()) {
            sb.append("  (Syntax : ").append(this.value.getTypeName()).append(")\n");
            sb.append("Value : ").append(this.value.toString());
        }
        else {
            sb.append("\nValue Exception : ").append(this.getValueStatusLegend());
        }
        return sb.toString();
    }
    
    private void setValueValid() {
        if (this.value == SnmpVarBind.endOfMibView) {
            this.status = 5;
        }
        else if (this.value == SnmpVarBind.noSuchObject) {
            this.status = 3;
        }
        else if (this.value == SnmpVarBind.noSuchInstance) {
            this.status = 4;
        }
        else {
            this.status = 2;
        }
    }
    
    private void handleLong(String s, int endIndex) throws NumberFormatException, SnmpStatusException {
        String substring;
        if (endIndex > 0) {
            substring = s.substring(0, endIndex);
        }
        else {
            substring = s;
        }
        Long.parseLong(substring);
    }
    
    static {
        statusLegend = new String[] { "Status Mapper", "Value not initialized", "Valid Value", "No such object", "No such Instance", "End of Mib View" };
        noSuchObject = new SnmpNull(128);
        noSuchInstance = new SnmpNull(129);
        endOfMibView = new SnmpNull(130);
    }
}
